<?php
/**
 *  BBClone Stats Renderer, visualy renders statistics generated by BBClone (http://www.bbclone.de/)
 *  Copyright (C) 2007 Martin Plicka
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
 
/**
 * URL parameters
 *   type: specifies sort of data to display
 *         browser - top10 browsers
 *         os      - top10 users' operating systems
 *         page    - top10 visited pages
 *         key     - top10 searched keywords
 *          
 * example: http://www.yoursite.com/stats.php?type=browser
 * 
 * READ readme.txt for more information.   
 */  
 


/**
 * SETTINGS for basic users. Advanced ones can edit what they want :-)
 */
 
// bbclone root directory, you must keep it actual
$BBCLONE_DIR = "./bbclone/";

//font source, TTF required. At this version, only monospace fonts give best look
$FONT = "./data/liberation_mono.ttf";

//initial angle for graph drawing, 0 means East, it uses clockwise direction
$ANGLE = 0;

//name for others item, that aren't in top10
$OTHERS_NAME = "- Ostatn -";

/**
 * END OF SETTINGS
 */


/********************************************************************************************************/


// includes $access array which contains all of statistics
require_once "$BBCLONE_DIR/var/access.php";

$type = isset($_GET["type"])?$_GET["type"]:"browser";


/**
 * For developers:
 * This array contain data to render, with keys and values as below:
 *  key = name that will be printed out
 *  value = number of hits
 * example:
 *  $data["Firefox"] = 5;
 *  $data["Explorer"] = 3;
 *  - means that there was 5 visits with FF, and 3 visits with IE    
 */   
$data = Array();

switch ($type) {
  // displays top10 browsers used to view monitored pages
  case "browser":
                  // imports array for browser name translation
                  require_once "$BBCLONE_DIR/lib/browser.php";
     
                  foreach ($access["stat"]["browser"] as $key=>$value){
                     //decode HTML entities in names
                     $data[$browser[$key]["title"]] = $value;
                  }
                  break;
                  
  // displays top10 visited pages (in format specified by BBClone usage)
  case "page":
                  foreach ($access["page"] as $key=>$value) {
                     $data[html_entity_decode($key)] = $value["count"];                  
                  }
                  break;
                  
  //displays top10 keywords refered by search engines
  case "key":     
                  $data = $access["key"];
                  break;
  case "os":
                  // imports array for os name translation
                  require_once "$BBCLONE_DIR/lib/os.php";
     
                  foreach ($access["stat"]["os"] as $key=>$value){
                     //decode HTML entities in names
                     $data[$os[$key]["title"]] = $value;
                  }
                  break;
  default: exit;
}

//sort data ascending
arsort($data, SORT_NUMERIC);

//number of all hits
$sum = array_sum($data);

/**
 * Strips array to 10 items, makes top10.
 * If you increase the length of array by modifying this function call,
 * note you must allocate more colors below
 */ 
array_splice($data, 10);

// add -others- item if there were more than 10 items
$newsum = array_sum($data);

if ($newsum < $sum) $data["other"] = $data["other"] + ($sum-$newsum);
if (isset($data["other"])) {
  $data[$OTHERS_NAME] = $data["other"];
  unset($data["other"]);
}

//number of items to calculate image height
$count = count($data);


//image allocation
$img = imagecreatetruecolor(200, 220 + ($count * 20));

//basic colors
$bg = imagecolorallocate($img, 255, 255, 255);
$black = imagecolorallocate ($img, 0,0,0);

//background
imagefill($img, 0, 0, $bg);    

/** 
 * Preset colors for some browsers, countries, etc.
 * Keys must be the same as in $data array calculated from stats. 
**/
switch ($type) {
 case "browser":
      $preset_colors["browser"]["Firefox"] = imagecolorallocate($img, 235, 150, 0);
      $preset_colors["browser"]["Explorer"] = imagecolorallocate($img, 50, 120, 255);
      $preset_colors["browser"]["Opera"] = imagecolorallocate($img, 220, 0, 0);
 break;
}

//color for "others"
$preset_colors[$type][$OTHERS_NAME] = imagecolorallocate($img, 210,210,210);


/**
 * standard colors
 */ 
$colors[]=imagecolorallocate($img, 0x66, 0xcc, 0);
$colors[]=imagecolorallocate($img, 0x33, 0xcc, 0xcc);
$colors[]=imagecolorallocate($img, 0xff, 0xff, 0x88);
$colors[]=imagecolorallocate($img, 0xff, 0x99, 0x66);
$colors[]=imagecolorallocate($img, 0x99, 0, 0);
$colors[]=imagecolorallocate($img, 0xaa, 0x77, 0xdd);
$colors[]=imagecolorallocate($img, 0xcc, 0xff, 0x33);
$colors[]=imagecolorallocate($img, 0x99, 0xff, 0xff);
$colors[]=imagecolorallocate($img, 0x33, 0x66, 0x00);
$colors[]=imagecolorallocate($img, 0xcc, 0x66, 0x00);



$i=0;
$color_index=0;
$begin = 0.0+$ANGLE;
$end = 0;

foreach($data as $key=>$value) {
  //select preset color (by the key name) or standard color
  $color = isset($preset_colors[$type][$key])
           ? $preset_colors[$type][$key]
           : $color = $colors[$color_index++]
         ;
  
  $end = $value/$sum * 360 + $begin;
  
  $name = str_pad( $key, 16);
  $hits = str_pad( $value, 5, " ", 0);
  $ratio = str_pad( sprintf("%.2f",$value/$sum*100)."%", 7, " ", 0);
  
  imagettftext($img, 8, 0, 19, 231+($i*20), $black, "$FONT",$name);
  imagettftext($img, 8, 0, 119, 231+($i*20), $black, "$FONT",$hits);
  imagettftext($img, 8, 0, 155, 231+($i*20), $black, "$FONT",$ratio);
  
  imagefilledrectangle($img, 2, 220+($i*20), 15, 233+($i*20), $color);
  imagerectangle($img, 2, 220+($i*20), 15, 233+($i*20), $black);
  
  //rounds precision fault of last item
  if ($i == $count-1) $end = $ANGLE + 360;
  
  //fills the pie slice
  imagefilledarc ( $img, 100, 100, 191, 191, $begin, $end, $color, IMG_ARC_PIE);
  
  //makes border
  imagefilledarc ( $img, 100, 100, 191, 191, $begin, $end, $black, IMG_ARC_NOFILL | IMG_ARC_EDGED);

  $begin = $end;
  $i++;
}

//flushes the image out
header("Content-Disposition: filename=\"stats_$type.png\"");
header("Content-type: image/png");
imagepng($img);

?>
